package me.xiaoyan.point.api.controller;

import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.pojo.OrderInfo;
import me.xiaoyan.point.api.pojo.UserInfo;
import me.xiaoyan.point.api.pojo.vo.CreateOrderData;
import me.xiaoyan.point.api.pojo.vo.OrderQueryParam;
import me.xiaoyan.point.api.pojo.vo.PageDataResult;
import me.xiaoyan.point.api.service.GoodsService;
import me.xiaoyan.point.api.service.OrderInfoService;
import me.xiaoyan.point.api.util.OrderStatus;
import me.xiaoyan.point.api.util.QueryWrapperUtil;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import java.util.HashMap;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 订单表(OrderInfo)表控制层
 *
 * @author makejava
 * @since 2022-11-24 09:32:38
 */
@Slf4j
@RestController
@RequestMapping("shop/order")
public class ShopOrderInfoController {
    @Resource
    private OrderInfoService orderInfoService;
    @Resource
    private GoodsService goodsService;
    @Resource
    private StringRedisTemplate stringRedisTemplate;

    // 是否已经没有库存
    // TODO 优化并发加锁
    private HashMap<Long, Boolean> stockOutMap = new HashMap<>();
    private static final String CACHE_STOCK_KEY = "goods:stock:";

    private String cacheKey(long gid) {
        return CACHE_STOCK_KEY + gid;
    }

    @PostConstruct // 初始化执行一次
    private void initGoodsStockCache() {
        //TODO 应该定时更新缓存数据
        goodsService.queryAllGoodsIdAndStock().forEach(g -> {
            log.info("缓存 id:{} stock:{} ", g.getId(), g.getStock());
            if (g.getStock() == 0) {
                // 原本就没有库存
                stockOutMap.put(g.getId(), true);
            }
            // 缓存库存
            stringRedisTemplate.opsForValue().set(cacheKey(g.getId()), g.getStock().toString());
        });
    }

    @PostMapping("create")
    public OrderInfo create(@Validated @RequestBody CreateOrderData data) {
        if (data.getGoodsId() <= 0 || data.getBuyCount() <= 0) {
            throw BizException.create("订单参数不正确");
        }
        int uid = StpUtil.getLoginIdAsInt();
        // 如果限制用户只能购买的数量，可以添加一个map记录用户的请求数
        int size = stockOutMap.size();
        Long buyId = (long) data.getGoodsId();
        //1.内存判断
        if (size > 0 && stockOutMap.containsKey(buyId) && stockOutMap.get(buyId)) {
            throw BizException.create("库存不足");
        }
        //2.缓存（redis）判断
        long count = stringRedisTemplate.opsForValue().decrement(cacheKey(buyId), data.getBuyCount());
        if (count < 0) {
            log.info("stock count ===>" + count);
            // 对缓存进行库存 还原
            stringRedisTemplate.opsForValue().increment(cacheKey(buyId), data.getBuyCount()); //
            throw BizException.create("库存不足");
        }
        if (count == 0) {
            // 此时库存没有了 , 保存到已买完的对象
            stockOutMap.put(buyId, true);
        }
        //3.数据库
        try {
            return orderInfoService.create(uid, data);
        } catch (BizException e) {
            //下单失败 对缓存进行库存 还原
            stringRedisTemplate.opsForValue().increment(cacheKey(data.getGoodsId()), data.getBuyCount()); //
            stockOutMap.put((long) data.getGoodsId(), false);
            throw e;
        }
    }

    // 只能查询自己关联的订单信息
    @PostMapping("/query")
    public Page<OrderInfo> query(@RequestBody OrderQueryParam param) {
        param.setUid(StpUtil.getLoginIdAsInt());
        return orderInfoService.queryByPage(param, true, false);
    }

    @GetMapping("/{id}")
    public OrderInfo get(@PathVariable("id") String id) {
        return orderInfoService.getOneByIdAndUid(id, StpUtil.getLoginIdAsInt());
    }

    private boolean updateStatus(String id, int status) {
        if (!StringUtils.hasText(id)) throw BizException.paramError();
        QueryWrapper<OrderInfo> q = QueryWrapperUtil.builder()
                .eq("id", id)
                .eq("uid", StpUtil.getLoginIdAsInt())
                .build();
        return orderInfoService.update(
                OrderInfo.builder()
                        .status(status)
                        .build(),
                q
        );
    }

    @PutMapping("/{id}/cancel")
    public boolean cancelOrder(@PathVariable("id") String id) {
        return updateStatus(id, OrderStatus.CANCELED);
    }

    @PutMapping("/{id}/delete")
    public boolean deleteOrder(@PathVariable("id") String id) {
        return updateStatus(id, OrderStatus.DELETE);
    }

}

