package me.xiaoyan.point.api.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.mapper.OrderInfoMapper;
import me.xiaoyan.point.api.pojo.Goods;
import me.xiaoyan.point.api.pojo.OrderInfo;
import me.xiaoyan.point.api.pojo.UserInfo;
import me.xiaoyan.point.api.pojo.dto.OrderStatus;
import me.xiaoyan.point.api.pojo.vo.CreateOrderData;
import me.xiaoyan.point.api.pojo.vo.OrderQueryParam;
import me.xiaoyan.point.api.service.*;
import me.xiaoyan.point.api.util.DataStatus;
import me.xiaoyan.point.api.util.OrderIdGenerator;
import me.xiaoyan.point.api.util.QueryWrapperUtil;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 订单表(OrderInfo)表服务实现类
 *
 * @author makejava
 * @since 2022-11-24 09:32:44
 */
@Service
public class OrderInfoServiceImpl extends ServiceImpl<OrderInfoMapper, OrderInfo> implements OrderInfoService {

    @Resource
    private UserInfoService userInfoService;
    @Resource
    private GoodsService goodsService;
    @Resource
    private PointRecordService pointRecordService;

    @Transactional
    @Override
    public OrderInfo create(int uid, CreateOrderData data) {
        // 1.查询用户信息
        final UserInfo user = userInfoService.getInfoById(uid);
        // 判断用户信息
        if (user == null) throw BizException.create("用户信息不存在");
        if (user.getStatus() != 1) throw BizException.create("用户状态不正确");
        // 2.查询商品信息
        final Goods goods = goodsService.getById(data.getGoodsId());
        if (goods == null) throw BizException.create("兑换的商品不存在");
        long now = new Date().getTime();
        if (goods.getOnlineTime().getTime() > now || goods.getOfflineTime().getTime() < now) {
            throw BizException.create("商品未上架或已下架");
        }
        if (goods.getStock() < data.getBuyCount()) throw BizException.create("商品存库不足");
        // 需要判断积分
        if (goods.getPrice() > 0 && user.getPointInfo().getTotalPoint() < goods.getPrice()) {
            throw BizException.create("积分不足，兑换失败");
        }
        // 判断购买数量的限制
        if (goods.getLimitCount() > 0 && buyHistoryCount(uid, data.getGoodsId()) + data.getBuyCount() > goods.getLimitCount()) {
            throw BizException.create("最多兑换" + goods.getLimitCount() + "件");
        }
        // 扣减积分
        pointRecordService.record(uid, 0 - goods.getPrice(), "兑换商品 " + goods.getTitle());
        // 3.减库存
        if (!goodsService.deductStock(data.getGoodsId(), data.getBuyCount())) {
            throw BizException.create("商品库存不足");
        }
        // 4.创建订单
        OrderInfo orderInfo = OrderInfo.builder()
                .id(OrderIdGenerator.next())
                .gid((long) data.getGoodsId())
                .uid(uid)
                .orderTitle(goods.getTitle())
                .count(data.getBuyCount())
                .price(goods.getPrice())
                // 默认为待确认
                .status(OrderStatus.NOT_CONFIRM)
                .build();
        if (save(orderInfo)) {
            return orderInfo;
        }
        throw BizException.create("创建订单失败");
    }

    @Override
    public Page queryByPage(OrderQueryParam param, boolean queryGoods, boolean queryUser) {

        final QueryWrapper q = QueryWrapperUtil.builder()
                .eq("id", param.getId())
                .eq("order_title", param.getTitle())
                .ge("create_time", param.getCreateTimeStart())
                .le("create_time", param.getCreateTimeEnd())
                .ne("status", DataStatus.DELETE)
                .build();
        // 根据状态筛选
        if (param.getStatus() != null && param.getStatus() > 0) {
            q.eq("status", param.getStatus());
        }
        q.orderByDesc("create_time");
        final Page<OrderInfo> page = getBaseMapper().selectPage(param.getPage(), q);
        page.getRecords().forEach(o -> {
            if (queryUser) {
                // 查询订单的归属用户
                o.setOwner(userInfoService.getById(o.getUid()));
            }
            if (queryGoods) {
                // 查询订单商品信息
                o.setGoods(goodsService.getById(o.getGid()));
            }
        });
        return page;
    }

    @Override
    public OrderInfo getOneByIdAndUid(String id, int uid) {
        QueryWrapper q = new QueryWrapper();
        q.eq("uid", uid);
        q.eq("id", id);
        final OrderInfo o = getOne(q);
        if(null != o){
            o.setGoods(goodsService.getById(o.getGid()));
        }
        return o;
    }

    public long buyHistoryCount(int uid, int gid) {
        QueryWrapper q = new QueryWrapper();
        q.eq("uid", uid);
        q.eq("gid", gid);
        q.ge("status", OrderStatus.CONFIRM); // 状态为2(待确认)和3(已完成)
        return count(q);
    }
}

