package me.xiaoyan.point.api.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import me.xiaoyan.point.api.pojo.vo.UserQueryParam;
import me.xiaoyan.point.api.util.DataStatus;
import me.xiaoyan.point.api.util.QueryWrapperUtil;
import me.xiaoyan.point.api.util.WechatDecryptDataUtil;
import me.xiaoyan.point.api.error.BizException;
import me.xiaoyan.point.api.mapper.UserInfoMapper;
import me.xiaoyan.point.api.pojo.Point;
import me.xiaoyan.point.api.pojo.UserInfo;
import me.xiaoyan.point.api.pojo.dto.CodeSessionData;
import me.xiaoyan.point.api.pojo.dto.WechatUserInfo;
import me.xiaoyan.point.api.pojo.vo.UserLoginData;
import me.xiaoyan.point.api.service.PointRecordService;
import me.xiaoyan.point.api.service.PointService;
import me.xiaoyan.point.api.service.UserInfoService;
import org.apache.hc.client5.http.classic.methods.HttpGet;
import org.apache.hc.client5.http.impl.classic.CloseableHttpClient;
import org.apache.hc.client5.http.impl.classic.CloseableHttpResponse;
import org.apache.hc.client5.http.impl.classic.HttpClients;
import org.apache.hc.core5.http.io.entity.EntityUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.Serializable;
import java.util.List;

@Service
@Slf4j
public class UserInfoServiceImpl extends ServiceImpl<UserInfoMapper, UserInfo>
        implements UserInfoService {
    private static final String LOG_TAG = "【USERSERVICE】";

    @Value("${wechat.app_id}")
    private String app_id;
    @Value("${wechat.app_secret}")
    private String app_secret;
    private static final String CODE2SESSION = "https://api.weixin.qq.com/sns/jscode2session";

    // 第一次登录的赠送积分
    @Value("${application.point.first_login}")
    private int firstLoginPoint;
    // 推荐人的赠送积分
    @Value("${application.point.recommend}")
    private int recommendPoint;


    @Resource
    private PointService pointService;
    @Resource
    private PointRecordService pointRecordService;

    public void firstLogin(UserInfo info, int recommendUid) {
        if (recommendUid > 0) {
            // 判断推荐者是否存在
            if (this.getById(recommendUid) != null) {
                // 存在则设置后续操作
                info.setParentId(recommendUid);
                // 增加推荐积分
                pointRecordService.record(
                        recommendUid,
                        recommendPoint,
                        "推荐新用户"
                );
            }
        }
        // 先新增用户基本信息
        save(info);
        // 新增积分信息
        Point point = Point.builder()
                .uid(info.getId())
                .expirePoint(0)
                .totalPoint(0)
                .validPoint(0)
                .build();
        pointService.save(point);
        // 新增积分记录
        pointRecordService.record(
                info.getId(),
                firstLoginPoint,
                "第一次登录赠送"
        );
    }


    @Transactional
    @Override
    public UserInfo login(UserLoginData data) {
        // 使用code换取登录session相关数据
        CodeSessionData sessionData = getOpenIdByCode(data.getCode());
        // 使用openid查询用户信息
        UserInfo userInfo = this.getBaseMapper().selectOneByOpenId(sessionData.getOpenid());
        // 判断用户信息是否存在
        if (userInfo == null) {
            // 先解码获取用户数据
            WechatUserInfo wechatUserInfo = WechatDecryptDataUtil.decryptData(
                    data.getEncryptedData(),
                    sessionData.getSessionKey(),
                    data.getIv()
            );
            userInfo = wechatUserInfo.getUserinfo().setOpenId(sessionData.getOpenid());
            //
            // 不存在走 第一次登录流程
            firstLogin(userInfo, data.getRecommend());
        }
        // 首先
        return userInfo;
    }

    private void updateUserBasicInfo(UserInfo userInfo) {
        try {
            this.getBaseMapper().updateById(userInfo);
        } catch (Exception e) {
            log.warn(LOG_TAG + "更新用户基本信息失败", e);
        }
    }

    /**
     * https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/login/auth.code2Session.html
     * 使用code换取openId
     */
    @SneakyThrows
    private CodeSessionData getOpenIdByCode(String code) {
        String url = CODE2SESSION + "?appid=" + app_id + "&secret=" + app_secret
                + "&js_code=" + code + "&grant_type=authorization_code";
        try (CloseableHttpClient client = HttpClients.createDefault()) {
            HttpGet get = new HttpGet(url);
            final CloseableHttpResponse response = (CloseableHttpResponse) client.execute(get);
            // 获取到session的json数据
            final String content = EntityUtils.toString(response.getEntity());
            Gson gson = new Gson();
            final CodeSessionData data = gson.fromJson(content, CodeSessionData.class);
            if (!StringUtils.hasLength(data.getOpenid()) || !StringUtils.hasLength(data.getSessionKey())
                    || (data.getErrcode() != null && data.getErrcode() != 0)) {
                System.out.println(content);
                throw BizException.create(1101, "换取session失败");
            }
            return data;
        }
    }

    public UserInfo getInfoById(Integer id) {
        final UserInfo userinfo = getById(id);
        if (userinfo != null) {
            userinfo.setPointInfo(pointService.getById(userinfo.getId()));
        }
        return userinfo;
    }

    @Override
    public UserInfo getInfoByCode(String code) {
        CodeSessionData sessionData = getOpenIdByCode(code);
        // 使用openid查询用户信息
        return this.getBaseMapper().selectOneByOpenId(sessionData.getOpenid());
    }


    private UserInfo getParent(int parentUid) {
        if (parentUid < 1) return null;
        return getBaseMapper().selectById(parentUid);
    }

    @Override
    public Page<UserInfo> queryByPage(UserQueryParam param) {
        QueryWrapper q = QueryWrapperUtil
                .builder()
                // 查询标题
                .eq("nickname", param.getNickname())
                .eq("open_id", param.getOpenId())
                .ne("status", DataStatus.DELETE)
                .build();
        Page<UserInfo> page = getBaseMapper().selectPage(param.getPage(), q);
        List<UserInfo> records = page.getRecords();
        if (records != null && records.size() > 0) {
            records.forEach(u -> {
                u.setPointInfo(pointService.getById(u.getId()));
                u.setParent(getParent(u.getParentId()));
            });

        }
        return page;
    }
}
